@@ -1,9 +1,11 @@
 #!/bin/sh -e
-#
+
 # Copyright (C) Internet Systems Consortium, Inc. ("ISC")
 #
+# SPDX-License-Identifier: MPL-2.0
+#
 # This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
+# License, v. 2.0.  If a copy of the MPL was not distributed with this
 # file, you can obtain one at https://mozilla.org/MPL/2.0/.
 #
 # See the COPYRIGHT file distributed with this work for additional
@@ -17,9 +19,9 @@ touch empty
 
 Z=cds.test
 
-keyz=`$KEYGEN -q -a RSASHA256 $Z`
-key1=`$KEYGEN -q -a RSASHA256 -f KSK $Z`
-key2=`$KEYGEN -q -a RSASHA256 -f KSK $Z`
+keyz=$($KEYGEN -q -a $DEFAULT_ALGORITHM $Z)
+key1=$($KEYGEN -q -a $DEFAULT_ALGORITHM -f KSK $Z)
+key2=$($KEYGEN -q -a $DEFAULT_ALGORITHM -f KSK $Z)
 
 idz=$(keyfile_to_key_id $keyz)
 id1=$(keyfile_to_key_id $key1)
@@ -42,8 +44,8 @@ convert() {
 	key=$1
 	n=$2
 	$DSFROMKEY -12 $key >DS.$n
-	grep ' 8 1 ' DS.$n >DS.$n-1
-	grep ' 8 2 ' DS.$n >DS.$n-2
+	grep " ${DEFAULT_ALGORITHM_NUMBER} 1 " DS.$n >DS.$n-1
+	grep " ${DEFAULT_ALGORITHM_NUMBER} 2 " DS.$n >DS.$n-2
 	sed 's/ IN DS / IN CDS /' <DS.$n >>CDS.$n
 	sed 's/ IN DS / IN CDS /' <DS.$n-1 >>CDS.$n-1
 	sed 's/ IN DS / IN CDS /' <DS.$n-2 >>CDS.$n-2
@@ -63,9 +65,9 @@ $PERL -we 'utime time, time - 7200, "DS.inplace" or die'
 
 mangle="$PERL mangle.pl"
 
-$mangle " IN DS $id1 8 1 " <DS.1 >DS.broke1
-$mangle " IN DS $id1 8 2 " <DS.1 >DS.broke2
-$mangle " IN DS $id1 8 [12] " <DS.1 >DS.broke12
+$mangle " IN DS $id1 ${DEFAULT_ALGORITHM_NUMBER} 1 " <DS.1 >DS.broke1
+$mangle " IN DS $id1 ${DEFAULT_ALGORITHM_NUMBER} 2 " <DS.1 >DS.broke2
+$mangle " IN DS $id1 ${DEFAULT_ALGORITHM_NUMBER} [12] " <DS.1 >DS.broke12
 
 sed 's/^/update add /
 $a\
@@ -121,14 +123,14 @@ $mangle '\s+IN\s+RRSIG\s+CDS .* '$idz' '$Z'\. ' \
 $mangle '\s+IN\s+RRSIG\s+CDS .* '$id1' '$Z'\. ' \
 	<sig.cds.1 >brk.rrsig.cds.ksk
 
-$mangle " IN CDS $id1 8 1 " <db.cds.1 |
+$mangle " IN CDS $id1 ${DEFAULT_ALGORITHM_NUMBER} 1 " <db.cds.1 |
 sign cds-mangled
 
-bad=`$PERL -le "print ($id1 ^ 255);"`
-sed 's/IN CDS '$id1' 8 1 /IN CDS '$bad' 8 1 /' <db.cds.1 |
+bad=$($PERL -le "print ($id1 ^ 255);")
+sed "s/IN CDS $id1 ${DEFAULT_ALGORITHM_NUMBER} 1 /IN CDS $bad ${DEFAULT_ALGORITHM_NUMBER} 1 /" <db.cds.1 |
 sign bad-digests
 
-sed '/IN CDS '$id1' 8 /p;s//IN CDS '$bad' 13 /' <db.cds.1 |
+sed "/IN CDS $id1 ${DEFAULT_ALGORITHM_NUMBER} /p;s//IN CDS $bad $ALTERNATIVE_ALGORITHM_NUMBER /" <db.cds.1 |
 sign bad-algos
 
 rm -f dsset-*
