@@ -12,9 +12,6 @@
  */
 
 /*
- * duration_fromtext initially taken from OpenDNSSEC code base.
- * Modified to fit the BIND 9 code.
- *
  * Copyright (c) 2009-2018 NLNet Labs.
  * All rights reserved.
  *
@@ -1033,12 +1030,12 @@ numlen(uint32_t num) {
  */
 void
 cfg_print_duration(cfg_printer_t *pctx, const cfg_obj_t *obj) {
-	char buf[CFG_DURATION_MAXLEN];
+	char buf[DURATION_MAXLEN];
 	char *str;
 	const char *indicators = "YMWDHMS";
 	int count, i;
 	int durationlen[7];
-	cfg_duration_t duration;
+	isccfg_duration_t duration;
 	/*
 	 * D ? The duration has a date part.
 	 * T ? The duration has a time part.
@@ -1090,7 +1087,7 @@ cfg_print_duration(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 	if (T) {
 		count++;
 	}
-	INSIST(count < CFG_DURATION_MAXLEN);
+	INSIST(count < DURATION_MAXLEN);
 
 	/* Now print the duration. */
 	for (i = 0; i < 6; i++) {
@@ -1119,7 +1116,7 @@ cfg_print_duration(cfg_printer_t *pctx, const cfg_obj_t *obj) {
 
 void
 cfg_print_duration_or_unlimited(cfg_printer_t *pctx, const cfg_obj_t *obj) {
-	cfg_duration_t duration;
+	isccfg_duration_t duration;
 
 	REQUIRE(pctx != NULL);
 	REQUIRE(obj != NULL);
@@ -1142,163 +1139,17 @@ cfg_obj_isduration(const cfg_obj_t *obj) {
 uint32_t
 cfg_obj_asduration(const cfg_obj_t *obj) {
 	REQUIRE(obj != NULL && obj->type->rep == &cfg_rep_duration);
-	uint32_t duration = 0;
-	duration += obj->value.duration.parts[6];	      /* Seconds */
-	duration += obj->value.duration.parts[5] * 60;	      /* Minutes */
-	duration += obj->value.duration.parts[4] * 3600;      /* Hours */
-	duration += obj->value.duration.parts[3] * 86400;     /* Days */
-	duration += obj->value.duration.parts[2] * 86400 * 7; /* Weaks */
-	/*
-	 * The below additions are not entirely correct
-	 * because days may very per month and per year.
-	 */
-	duration += obj->value.duration.parts[1] * 86400 * 31;	/* Months */
-	duration += obj->value.duration.parts[0] * 86400 * 365; /* Years */
-	return (duration);
-}
-
-static isc_result_t
-duration_fromtext(isc_textregion_t *source, cfg_duration_t *duration) {
-	char buf[CFG_DURATION_MAXLEN];
-	char *P, *X, *T, *W, *str;
-	bool not_weeks = false;
-	int i;
-
-	/*
-	 * Copy the buffer as it may not be NULL terminated.
-	 * Anyone having a duration longer than 63 characters is crazy.
-	 */
-	if (source->length > sizeof(buf) - 1) {
-		return (ISC_R_BADNUMBER);
-	}
-	/* Copy source->length bytes and NULL terminate. */
-	snprintf(buf, sizeof(buf), "%.*s", (int)source->length, source->base);
-	str = buf;
-
-	/* Clear out duration. */
-	for (i = 0; i < 7; i++) {
-		duration->parts[i] = 0;
-	}
-
-	/* Every duration starts with 'P' */
-	P = strpbrk(str, "Pp");
-	if (P == NULL) {
-		return (ISC_R_BADNUMBER);
-	}
-
-	/* Record the time indicator. */
-	T = strpbrk(str, "Tt");
-
-	/* Record years. */
-	X = strpbrk(str, "Yy");
-	if (X != NULL) {
-		duration->parts[0] = atoi(str + 1);
-		str = X;
-		not_weeks = true;
-	}
-
-	/* Record months. */
-	X = strpbrk(str, "Mm");
-
-	/*
-	 * M could be months or minutes. This is months if there is no time
-	 * part, or this M indicator is before the time indicator.
-	 */
-	if (X != NULL && (T == NULL || (size_t)(X - P) < (size_t)(T - P))) {
-		duration->parts[1] = atoi(str + 1);
-		str = X;
-		not_weeks = true;
-	}
-
-	/* Record days. */
-	X = strpbrk(str, "Dd");
-	if (X != NULL) {
-		duration->parts[3] = atoi(str + 1);
-		str = X;
-		not_weeks = true;
-	}
-
-	/* Time part? */
-	if (T != NULL) {
-		str = T;
-		not_weeks = true;
-	}
-
-	/* Record hours. */
-	X = strpbrk(str, "Hh");
-	if (X != NULL && T != NULL) {
-		duration->parts[4] = atoi(str + 1);
-		str = X;
-		not_weeks = true;
-	}
-
-	/* Record minutes. */
-	X = strpbrk(str, "Mm");
-
-	/*
-	 * M could be months or minutes. This is minutes if there is a time
-	 * part and the M indicator is behind the time indicator.
-	 */
-	if (X != NULL && T != NULL && (size_t)(X - P) > (size_t)(T - P)) {
-		duration->parts[5] = atoi(str + 1);
-		str = X;
-		not_weeks = true;
-	}
-
-	/* Record seconds. */
-	X = strpbrk(str, "Ss");
-	if (X != NULL && T != NULL) {
-		duration->parts[6] = atoi(str + 1);
-		str = X;
-		not_weeks = true;
-	}
-
-	/* Or is the duration configured in weeks? */
-	W = strpbrk(buf, "Ww");
-	if (W != NULL) {
-		if (not_weeks) {
-			/* Mix of weeks and other indicators is not allowed */
-			return (ISC_R_BADNUMBER);
-		} else {
-			duration->parts[2] = atoi(str + 1);
-			str = W;
-		}
-	}
-
-	/* Deal with trailing garbage. */
-	if (str[1] != '\0') {
-		return (ISC_R_BADNUMBER);
-	}
-
-	return (ISC_R_SUCCESS);
+	return isccfg_duration_toseconds(&(obj->value.duration));
 }
 
 static isc_result_t
 parse_duration(cfg_parser_t *pctx, cfg_obj_t **ret) {
 	isc_result_t result;
 	cfg_obj_t *obj = NULL;
-	cfg_duration_t duration;
+	isccfg_duration_t duration;
 
-	duration.unlimited = false;
-
-	if (toupper((unsigned char)TOKEN_STRING(pctx)[0]) == 'P') {
-		result = duration_fromtext(&pctx->token.value.as_textregion,
-					   &duration);
-		duration.iso8601 = true;
-	} else {
-		uint32_t ttl;
-		result = dns_ttl_fromtext(&pctx->token.value.as_textregion,
-					  &ttl);
-		/*
-		 * With dns_ttl_fromtext() the information on optional units.
-		 * is lost, and is treated as seconds from now on.
-		 */
-		for (int i = 0; i < 6; i++) {
-			duration.parts[i] = 0;
-		}
-		duration.parts[6] = ttl;
-		duration.iso8601 = false;
-	}
+	result = isccfg_parse_duration(&pctx->token.value.as_textregion,
+				       &duration);
 
 	if (result == ISC_R_RANGE) {
 		cfg_parser_error(pctx, CFG_LOG_NEAR,
@@ -1346,7 +1197,7 @@ cfg_parse_duration_or_unlimited(cfg_parser_t *pctx, const cfg_type_t *type,
 				cfg_obj_t **ret) {
 	isc_result_t result;
 	cfg_obj_t *obj = NULL;
-	cfg_duration_t duration;
+	isccfg_duration_t duration;
 
 	UNUSED(type);
 
